<?php

namespace QUI\ERP\Coupons\Products;

use QUI;
use QUI\ERP\Order\AbstractOrder;
use QUI\ERP\Products\Product\Product;
use QUI\ERP\Discount\Discount;
use QUI\ERP\Discount\Handler as DiscountHandler;

/**
 * Class Handler
 *
 * Handles coupon code generation from orders / products.
 */
class Handler
{
    /**
     * Create coupon codes from an order
     *
     * @param Product $Product
     */
    public static function createCouponCodeFromProduct(Product $Product)
    {

    }

    /**
     * Create and/or updates the discount that is associated with a coupon code product.
     *
     * @param Product $Product
     * @return void
     *
     * @throws QUI\Exception
     */
    public static function updateProductDiscount(Product $Product): void
    {
        $Discount = self::getProductDiscount($Product);

        if (!$Discount) {
            $Discount = self::createProductDiscount($Product);
        }

        $discountAmount = $Product->getFieldValue(CouponProductType::PRODUCT_FIELD_ID_COUPON_AMOUNT);

        if (empty($discountAmount)) {
            $discountAmount = $Product->getPrice()->getValue();
        }

        $Discount->setAttributes([
            'discount' => $discountAmount
        ]);

        $Discount->update();
    }

    /**
     * Creates a new discount for all coupon codes generated by the product.
     *
     * @param Product $Product
     * @return Discount
     *
     * @throws QUI\Exception
     */
    protected static function createProductDiscount(Product $Product): Discount
    {
        $Handler = DiscountHandler::getInstance();

        $discountAmount = $Product->getFieldValue(CouponProductType::PRODUCT_FIELD_ID_COUPON_AMOUNT);

        if (empty($discountAmount)) {
            $discountAmount = $Product->getPrice()->getValue();
        }

        /** @var Discount $NewDiscount */
        $NewDiscount = $Handler->createChild([
            'active'        => 1,
            'discount'      => $discountAmount,
            'discount_type' => DiscountHandler::DISCOUNT_TYPE_CURRENCY
        ]);

        $L             = new QUI\Locale();
        $discountTitle = [];

        foreach (QUI::availableLanguages() as $lang) {
            $L->setCurrent($lang);

            $discountTitle[$lang] = $L->get('quiqqer/coupons', 'Discount.default_title.product', [
                'productId'    => $Product->getId(),
                'productTitle' => $Product->getTitle($L)
            ]);
        }

        \QUI\Translator::update(
            'quiqqer/discount',
            'discount.'.$NewDiscount->getId().'.title',
            'quiqqer/discount',
            $discountTitle
        );

        \QUI\Translator::publish('quiqqer/discount');

        QUI::getDataBase()->update(
            QUI\ERP\Products\Utils\Tables::getProductTableName(),
            [
                'couponDiscountId' => $NewDiscount->getId()
            ],
            [
                'id' => $Product->getId()
            ]
        );

        return $NewDiscount;
    }

    /**
     * Get the discount that is associated with a coupon code product.
     *
     * @param Product $Product
     * @return Discount|false - Discount or false if no discount created yet
     *
     * @throws QUI\Exception
     */
    public static function getProductDiscount(Product $Product)
    {
        $result = QUI::getDataBase()->fetch([
            'select' => ['couponDiscountId'],
            'from'   => QUI\ERP\Products\Utils\Tables::getProductTableName(),
            'where'  => [
                'id' => $Product->getId()
            ]
        ]);

        if (empty($result)) {
            return false;
        }

        /** @var Discount $Discount */
        $Discount = DiscountHandler::getInstance()->getChild($result[0]['couponDiscountId']);

        return $Discount;
    }
}
